/*** FILEHEADER ****************************************************************
 *
 *   FILENAME:    demo.c
 *   DATE:        04.08.2004
 *   AUTHOR:       (c) Christian Stadler
 *
 * DESCRIPTION: RFM12 wireless FSK transceiver module demo program.
 *              To be used with PIC18F4XX demo board.
 *              RFM12 module connection to PIC18F4XX demo board:
 *                  RD7 = SCK
 *                  RD6 = SDI
 *                  RD5 = nSel
 *                  RD4 = SDO
 *                  RD3 = IRQ
 *
 ******************************************************************************/

/*** HISTORY OF CHANGE *********************************************************
 *
 *   $Log: /pic/rfm12demo/rfm12demo.c $
 * 
 * 8     1.11.10 13:42 Stadler
 * - make code compilable with C18 compiler and CCS compiler
 * 
 * 7     1.11.10 12:17 Stadler
 * - moved RFM12 pin configuration into rfm12conf.h
 * 
 * 6     31.10.10 17:27 Stadler
 * - separeated RFM12 driver code into separate file
 * 
 * 5     31.10.10 14:16 Stadler
 * - transmit 2 dummy bytes at the end of data transfer
 * 
 * 4     30.10.10 22:18 Stadler
 * - fixed RFM12_Init
 * - changed LED indication: Receiver keeps LED1 on
 * 
 * 3     30.10.10 21:39 Stadler
 * - increased init delay
 * - removed warnings
 * 
 * 2     30.10.10 20:23 Stadler
 * - updated code documentation
 * 
 * 1     30.10.10 18:49 Stadler
 * - RFM12 wireless FSK transceiver module demo
 *
 ******************************************************************************/

/* make compiler case sensitive */
#pragma case

/* To use the demo, 1 receiver and 1 transmitter is required. */
/* To generate HEX file for the receiver uncomment the #define MODE_RECEIVER */
/* and comment the #define MODE_TRANSMITTER. */
/* To generate HEX file for the receiver uncomment the #define */
/* MODE_TRANSMITTER and comment the #define MODE_RECEIVER. */

#define MODE_RECEIVER
//#define MODE_TRANSMITTER



/* compiler configuration */
#include "rfm12demo.h"

/* data type definitions */
#include "types.h"


/* RFM12 driver */
#include "rfm12.h"
/* CCS compiler specific: also drive C file needs to be included */
#if (defined(__PCB__) || defined(__PCH__) || defined(__PCM__))
#include "rfm12.c"
#endif /* #if (defined(__PCB__) || defined(__PCH__) || defined(__PCM__)) */


/* Onboard LEDs on PIC18F4XX demo board */
/* C18 compiler specific */
#if defined(__18CXX)
#define LED1_On()   LATCbits.LATC0 = 1 // PIN_C0
#define LED1_Off()  LATCbits.LATC0 = 0 // PIN_C0
#define LED2_On()   LATCbits.LATC1 = 1 // PIN_C1
#define LED2_Off()  LATCbits.LATC1 = 0 // PIN_C1
#endif /* #if defined(__18CXX) */

/* CCS compiler specific */
#if (defined(__PCB__) || defined(__PCH__) || defined(__PCM__))
#define LED1_On()       output_high(PIN_C0);
#define LED1_Off()      output_low(PIN_C0);
#define LED2_On()       output_high(PIN_C1);
#define LED2_Off()      output_low(PIN_C1);
#endif /* #if (defined(__PCB__) || defined(__PCH__) || defined(__PCM__)) */



/*******************************************************************************
 * FUNCTION:   void init(void)
 * PURPOSE:    Initializes PIC and external HW.
 ******************************************************************************/
void init(void)
{
    /* C18 compiler specific */
    #if defined(__18CXX)
    ADCON1 |= 0x0F;
    
    /* init LED pins */
    LATC &= 0xFC;
    TRISC &= 0xFC;
    
    /* baud rate formula for BRGH = 0: */
    /* BaudRate = FOSC/(64 (X + 1)) */
    /* => BaudRate (64 (X + 1)) = FOSC */
    /* => 64 (X + 1) = FOSC / BaudRate */
    /* => X = (FOSC / BaudRate) / 64 - 1 */
    /* => X = SPBRG = /32000000 / 19200) / 64 - 1 = 25.04166 => 25 */
    OpenUSART(USART_TX_INT_OFF &    /* disables TX interrupt */
              USART_RX_INT_OFF &    /* disables RX interrupt */
              USART_ASYNCH_MODE &   /* sets it in asynchronous mode */
              USART_EIGHT_BIT &     /* sets to use 8-bit data mode */
              USART_CONT_RX &       /* sets the port in continues receive mode */
              USART_BRGH_LOW,       /* uses the low speed Baud rate formula */
              25);                  /* SPBRG, sets Baud to 19200 BPS */
                      
    #endif /* #if defined(__18CXX) */
    

    /* CCS compiler specific */
    #if (defined(__PCB__) || defined(__PCH__) || defined(__PCM__))
    /* generated by PIC wizard */
    setup_adc_ports(NO_ANALOGS);
    setup_adc(ADC_OFF);
    setup_psp(PSP_DISABLED);
    setup_spi(FALSE);
    setup_wdt(WDT_OFF);
    setup_timer_0(RTCC_INTERNAL);
    setup_timer_1(T1_DISABLED);
    setup_timer_2(T2_DISABLED,0,1);
    setup_timer_3(T3_DISABLED|T3_DIV_BY_1);
    setup_comparator(NC_NC_NC_NC);
    #endif /* #if (defined(__PCB__) || defined(__PCH__) || defined(__PCM__)) */

    /* switch off LEDs */
    LED1_Off();
    LED2_Off();
}



/*******************************************************************************
 * FUNCTION:   void receive(void)
 * PURPOSE:    Receive data.
 ******************************************************************************/
void receive(void)
{
    static uint16 rxcount = 0;
    uint8 rxbuf[40];
    
    /* init RX buffer */
    memset(rxbuf, 0, sizeof(rxbuf));
    
    RFM12_RxData(rxbuf, 32);
    rxcount++;

    /* C18 compiler specific */
    #if defined(__18CXX)
    printf("%d: %s\n\r", rxcount, rxbuf);
    #endif

    /* CCS compiler specific */
    #if (defined(__PCB__) || defined(__PCH__) || defined(__PCM__))
    printf("%ld: %s\n\r", rxcount, rxbuf);
    #endif
}



/*******************************************************************************
 * FUNCTION:   void transmit(void)
 * PURPOSE:    Transmit data.
 ******************************************************************************/
void transmit(void)
{
    /*             0         1         2         3  */
    /*             01234567890123456789012345678901 */
    uint8 txbuf[]="*** Hello World! RFM12 Demo! ***";
    RFM12_TxData(txbuf, 32);
}


/*******************************************************************************
 * FUNCTION:   void main(void)
 * PURPOSE:    Main.
 ******************************************************************************/
void main(void)
{
    init();
    
    /* init RFM12 module */
    RFM12_Init();
    RFM12_SetFreq(RFM12FREQ(433.92)); /* Rx/Tx frequency 433.92MHz */
    RFM12_SetBandwidth(4, 1, 4);      /* 200kHz Bandwidth, -6dB LNA gain, DRSSI threshold: -79dBm */
    RFM12_SetBaud(19200);             /* 19200 baud */
    RFM12_SetPower(0, 6);             /* 1mW output powr, 120kHz frequency shift */

    /* show power up via LED on for 2 seconds */
    LED1_On();
    delay_ms(2000);
    LED1_Off();

    #ifdef MODE_RECEIVER
    /* keep LED1 on to be able to distinguish between receive and transmitter */
    LED1_On();
    #endif
    

    /* main loop */
    while (1)
    {
        #ifdef MODE_TRANSMITTER
        transmit();
        
        LED2_On();
        delay_ms(50);
        LED2_Off();
        delay_ms(2000);    
        #endif

        #ifdef MODE_RECEIVER
        receive();
        
        LED2_On();
        delay_ms(20);
        LED2_Off();
        #endif
    }
}

